unit SocketCommand;
{
 Composants TClientCMDSocket et TServerCMDSocket permettant l'envoi et l'analyse de commandes (avec ou sans arguments) entre
 clients et serveur, en mode asynchrone (sockets non bloquants).

 Ces composants sont drivs de TClientSocket et TServerSocket contenus dans le package "Composants Socket Borland" inclus dans
 ($Delphi)\Bin\dclsocketsXX.bpl (non install par dfaut)

 TClientCMDSocket
 -----------------

 Proprits :
 - CaseSensitive (bool) : indique si la rception des commandes transmises par le serveur doit tenir compte de la casse
 - Commands (TStrings) : indique la liste des commandes autorises par TClientCMDSocket

 Fonction :
 - SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString
   Command : Commmande  transmettre au serveur
   Args : Liste optionnelle d'arguments de types divers (chanes, entiers, dcimaux, boolen, streams/tableaux).
   TimeOut (facultatif) : Dlai maximum (en ms) de rponse du serveur  cette commande, dlai au-del duquel un vnement OnCommandTimeOut est dclench
                          Mettre 0 pour ne pas dfinir de dlai.
   Reply_ID (facultatif) : Indique l'identifiant du message auquel on rpond
   ID (facultatif) : Indique l'identifiant qu'on souhaite dfinir pour le message transmis
   Si ID n'est pas dfini, un GUID est gnr par l'application.

   La fonction retourne l'ID du message.

 Evnements :
 - OnCommand(Sender: TClientCMDSocket ; CommandEx: TCommandUser)
   Se produit  la rception d'une commande transmise par le serveur.
   Soit cette commande fait partie de la liste des Commands, soit elle a t accepte lors de l'vnement OnCommandError
   Le paramtre CommandEx contient la commande, l'ID du message, le Reply_ID du message, et la liste des arguments (format Variant)

 - OnCommandError(Sender: TClientCMDSocket ; CommandEx: TCommandUser ; ErrTyp: TCMDSCError ; var Continue: bool)
   Se produit  la rception d'une commande (CommandEx) incorrecte :
    - soit car la commande est inconnue (ErrTyp=CMDSC_COMMAND_UNKNOWN)
    - soit car ses paramtres sont incorrects (ErrTyp=CMDSC_ILLEGAL_PARAMETER)
   En dfinissant Continue  true, on autorise la poursuite de l'analyse de la commande.
   Si on dfinit Continue  false, l'analyse de la commande est abandonne et la suite de la trame est analyse

 - OnCommandTimeOut(Sender: TClientCMDSocket ; Command, ID: AnsiString ; TimeOut: cardinal)
   Se produit lorsqu'aucune rponse n'a t apporte par le serveur aprs l'envoi d'une commande via SendCommand avec un TimeOut
   Command, ID et TimeOut rappellent les caractristiques principales de la commande reste sans rponse.


 TServerCMDSocket
 -----------------

 Proprits :
 - CaseSensitive (bool) : indique si la rception des commandes transmises par les clients doit tenir compte de la casse
 - Commands (TStrings) : indique la liste des commandes autorises par TServerCMDSocket

 Chaque socket actif du TServerCMDSocket dispose de la fonction :
 - SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString
   Command : Commmande  transmettre au client
   Args : Liste optionnelle d'arguments de types divers (chanes, entiers, dcimaux, boolen, streams/tableaux).
   TimeOut (facultatif) : Dlai maximum (en ms) de rponse du client  cette commande, dlai au-del duquel un vnement OnClientCommandTimeOut est dclench
                          Mettre 0 pour ne pas dfinir de dlai.
   Reply_ID (facultatif) : Indique l'identifiant du message auquel on rpond
   ID (facultatif) : Indique l'identifiant qu'on souhaite dfinir pour le message transmis
   Si ID n'est pas dfini, un GUID est gnr par l'application.

   La fonction retourne l'ID du message.

 Evnements :
 - OnClientCommand(Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; CommandEx: TCommandUser)
   Se produit  la rception d'une commande transmise sur le Socket.
   Soit cette commande fait partie de la liste des Commands, soit elle a t accepte lors de l'vnement OnClientCommandError
   Le paramtre CommandEx contient la commande, l'ID du message, le Reply_ID du message, et la liste des arguments (format Variant)

 - Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; CommandEx: TCommandUser ; ErrTyp: TCMDSCError ; var Continue: bool)
   Se produit  la rception d'une commande (Command) incorrecte sur le Socket :
    - soit car la commande est inconnue (ErrTyp=CMDSC_COMMAND_UNKNOWN)
    - soit car ses paramtres sont incorrects (ErrTyp=CMDSC_ILLEGAL_PARAMETER)
   En dfinissant Continue  true, on autorise la poursuite de l'analyse de la commande.
   Si on dfinit Continue  false, l'analyse de la commande est abandonne et la suite de la trame est analyse

 - OnClientCommandTimeOut(Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; Command, ID: AnsiString ; TimeOut: cardinal)
   Se produit lorsqu'aucune rponse n'a t apporte par le client aprs l'envoi d'une commande via SendCommand avec un TimeOut
   Command, ID et TimeOut rappellent les caractristiques principales de la commande reste sans rponse.


 Routines de conversion :
 
 function VarArrayDim(Data: Variant): integer;
   Renvoit la taille du tableau Variant (1 dimension) donn en paramtre

 function StreamToVariant(Stream: TStream ; Count: int64): Variant;
   Convertit le Stream en Variant sur les Count prochains bytes.

 function ArrayOfByteToVariant(Buf: array of byte): Variant;
   Convertit le tableau de bytes en Variant

 procedure VariantToStream(Data: Variant ; Stream: TStream);
   Convertit le Variant en Stream

 function VariantToArrayOfByte(Data: Variant): array of byte;
   Convertit le tableau de bytes en Variant

 Test sous Delphi 7 et XE.

 Columbo pour phidels.com

 version 1.2 (03/05/2013) : - Ajout de la fonction VarArrayDim
                            - Amliorations mineures
 version 1.1 (03/11/2012) : Modification des paramtres de l'vnement d'erreur (client & serveur)
 version 1.0 (03/09/2012) : Premire version
}
interface

uses Windows, Messages, WinSock, Classes, ScktComp, SysUtils, Math, Variants, ExtCtrls;

const EOL: AnsiString=#13#10;
      Arg_Marker: array[0..4] of AnsiChar=('S', 'I', 'D', 'B', 'A');
      WM_DELETE_TIMER=WM_USER+1024;
      WM_COMMAND=WM_USER+1025;
      varUString  = $0102;
      MAX_ARRAY_SIZE=20*1024*1024; // Taille limite d'un tableau/fichier (20 Mo)

type
  TArrayOfChar = array of Ansichar;
  TArrayOfByte = array of byte;

  TArgType=(AT_STRING, AT_INT64, AT_DOUBLE, AT_BOOL, AT_ARRAY);

  TArgument = record
    Data: TArrayOfChar;
    case Typ: TArgType of
    AT_ARRAY: (Size: integer);
    AT_STRING, AT_INT64, AT_DOUBLE, AT_BOOL: ();
    end;

  TCommand = record
   State: (CS_COMMAND, CS_ID, CS_REPLY_ID, CS_STRUCT, CS_ARGUMENT);
   Command, ID, Reply_ID, Struct: AnsiString;
   Argument: array of TArgument;
   Argument_Index: integer;
  end;

  TArgUser = record
    Typ: TArgType;
    Data: Variant;
  end;

  TCommandUser = record
    Command, ID, Reply_ID: AnsiString;
    Argument: array of TArgUser;
  end;

  TSocketState=(ssText, ssFile);

  TCMDSCError=(CMDSC_COMMAND_UNKNOWN, CMDSC_ILLEGAL_PARAMETER);

  TClientCMDSocket = class;
  TServerCMDSocket = class;
  TServerClientCMDSocket = class;

  AnsiStr=type AnsiString;

  TCustomCMDSocketCommandEvent = procedure (Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommandUser) of object;
  TCustomCMDSocketCommandErrorEvent = procedure (Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommand ; var Continue: bool) of object;
  TCustomCMDSocketCommandTimeOut = procedure (Sender: TObject ; Socket: TCustomWinSocket; Command, ID: AnsiStr ; TimeOut: cardinal) of object;

  TServerCMDSocketCommandEvent = procedure (Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; CommandEx: TCommandUser) of object;
  TServerCMDSocketCommandErrorEvent = procedure (Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; CommandEx: TCommandUser ; ErrTyp: TCMDSCError ; var Continue: bool) of object;
  TServerCMDSocketCommandTimeOut = procedure (Sender: TServerCMDSocket ; Socket: TServerClientCMDSocket; Command, ID: AnsiStr ; TimeOut: cardinal) of object;

  TClientCMDSocketCommandEvent = procedure (Sender: TClientCMDSocket ; CommandEx: TCommandUser) of object;
  TClientCMDSocketCommandErrorEvent = procedure (Sender: TClientCMDSocket ; CommandEx: TCommandUser ; ErrTyp: TCMDSCError ; var Continue: bool) of object;
  TClientCMDSocketCommandTimeOut = procedure (Sender: TClientCMDSocket ; Command, ID: AnsiStr ; TimeOut: cardinal) of object;
  TClientCMDSocketConnectTimeOut = procedure (Sender: TClientCMDSocket ; TimeOut: cardinal) of object;

  TTimeOutTimer = class(TTimer)
  private
    FCommand, FID: AnsiString;
    FParentHandle: HWND;
  public
    property Command: AnsiString read FCommand write FCommand;
    property ID: AnsiString read FID write FID;
    property ParentHandle: HWND read FParentHandle write FParentHandle;
  end;

  TTimerList = class(TList)
  private
    FWindowHandle: HWND;
  public
    constructor Create;
    destructor Destroy; override;
    procedure ClearTimers(Index: integer=-1);
    procedure WndProc(var Msg: TMessage); virtual;
    property Handle: HWND read FWindowHandle;
  end;

  TCommandParser = class(TObject)
  private
    FWindowHandle: HWND;
    FCustomCMDSocketCommandIncoming, FCustomCMDSocketCommandStructError: TCustomCMDSocketCommandErrorEvent;
    FCustomCMDSocketCommandEvent: TCustomCMDSocketCommandEvent;
    FCurrentCommand: TCommand;
    FSocket: TCustomWinSocket;
    CommandQueued: array of TCommandUser;
    procedure RemoveCommandQueued(Index: integer=-1);
  protected
    TimerList: TTimerList;
  public
    constructor Create(ASocket: TCustomWinSocket);
    destructor Destroy; override;
    procedure InitCommand;
    procedure WndProc(var Msg: TMessage); virtual;
    procedure Parse(var Buffer: TArrayOfChar);
    property CurrentCommand: TCommand read FCurrentCommand write FCurrentCommand;
    property Socket: TCustomWinSocket read FSocket;
    property OnCommandReceive: TCustomCMDSocketCommandErrorEvent read FCustomCMDSocketCommandIncoming write FCustomCMDSocketCommandIncoming;
    property OnCommandStructError: TCustomCMDSocketCommandErrorEvent read FCustomCMDSocketCommandStructError write FCustomCMDSocketCommandStructError;
    property OnCommand: TCustomCMDSocketCommandEvent read FCustomCMDSocketCommandEvent write FCustomCMDSocketCommandEvent;
  end;

  TClientCMDSocket = class(TCustomSocket)
  private
    FClientCMDSocketCommandTimeOut: TClientCMDSocketCommandTimeOut;
    FClientCMDSocketCommandEvent: TClientCMDSocketCommandEvent;
    FClientCMDSocketCommandErrorEvent: TClientCMDSocketCommandErrorEvent;
    FClientCMDSocketConnectTimeOut: TClientCMDSocketConnectTimeOut;
    FCommands: TStrings;
    FConnectTimer: TTimer;
  protected
    FClientSocket: TClientWinSocket;
    CommandParser: TCommandParser;
    function GetClientType: TClientType;
    procedure DoActivate(Value: Boolean); override;
    procedure SetCommands(Value: TStrings);
    function GetCaseSensitive: bool;
    procedure SetCaseSensitive(Value: bool);
    procedure OnTimeOutEvent(Sender: TObject);
    procedure OnCommandOK(Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommandUser); virtual;
    procedure OnCommandReceive(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool); virtual;
    procedure OnCommandStructError(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString;
    property Socket: TClientWinSocket read FClientSocket;
    property ClientType: TClientType read GetClientType;
    procedure Event(Socket: TCustomWinSocket; SocketEvent: TSocketEvent); override;
    procedure ConnectAndWaitForCommand(TimeOut: cardinal);
   published
    property Active;
    property Address;
    property CaseSensitive: bool read GetCaseSensitive write SetCaseSensitive default true;
    property Commands: TStrings read FCommands write SetCommands;
    property Host;
    property Port;
    property Service;
    property OnLookup;
    property OnConnecting;
    property OnConnect;
    property OnDisconnect;
    property OnCommand: TClientCMDSocketCommandEvent read FClientCMDSocketCommandEvent write FClientCMDSocketCommandEvent;
    property OnCommandError: TClientCMDSocketCommandErrorEvent read FClientCMDSocketCommandErrorEvent write FClientCMDSocketCommandErrorEvent;
    property OnCommandTimeOut: TClientCMDSocketCommandTimeOut read FClientCMDSocketCommandTimeOut write FClientCMDSocketCommandTimeOut;
    property OnConnectTimeOut: TClientCMDSocketConnectTimeOut read FClientCMDSocketConnectTimeOut write FClientCMDSocketConnectTimeOut;
    property OnError;
  end;

  TServerClientCMDSocket = class(TServerClientWinSocket)
  private
    FOwner: TServerCMDSocket;
  protected
    CommandParser: TCommandParser;
    procedure Event(Socket: TCustomWinSocket; SocketEvent: TSocketEvent); override;
    procedure OnCommandOK(Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommandUser); virtual;
    procedure OnCommandReceive(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool); virtual;
    procedure OnCommandStructError(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool); virtual;
    procedure OnTimeOutEvent(Sender: TObject); virtual;
  public
    constructor Create(AOwner: TServerCMDSocket; Socket: TSocket; ServerWinSocket: TServerWinSocket);
    destructor Destroy; override;
    function SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString;
  end;

  TServerWinCMDSocket= class(TServerWinSocket)
  private
     function GetConnections(Index: Integer): TServerClientCMDSocket;
  public
     property Connections[Index: Integer]: TServerClientCMDSocket read GetConnections;
  end;

  TServerCMDSocket = class(TCustomServerSocket)
  private
    FCommands: TStrings;
    FServerCMDSocketCommandTimeOut: TServerCMDSocketCommandTimeOut;
    FServerCMDSocketCommandEvent: TServerCMDSocketCommandEvent;
    FServerCMDSocketCommandErrorEvent: TServerCMDSocketCommandErrorEvent;
  protected
    procedure GetSocket(Sender: TObject; ASocket: TSocket; var ClientSocket: TServerClientWinSocket); virtual;
    function GetServerType: TServerType;
    function GetCaseSensitive: bool;
    procedure SetCaseSensitive(Value: bool);
    procedure SetCommands(Value: TStrings);
    function GetServerSocket: TServerWinCMDSocket;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Socket: TServerWinCMDSocket read GetServerSocket;
    property ServerType: TServerType read GetServerType;
  published
    property Active;
    property CaseSensitive: bool read GetCaseSensitive write SetCaseSensitive default true;
    property Commands: TStrings read FCommands write SetCommands;
    property Port;
    property Service;
    property ThreadCacheSize default 10;
    property OnListen;
    property OnAccept;
    property OnGetSocket;
    property OnClientConnect;
    property OnClientDisconnect;
    property OnClientWrite;
    property OnClientError;
    property OnClientCommand: TServerCMDSocketCommandEvent read FServerCMDSocketCommandEvent write FServerCMDSocketCommandEvent;
    property OnClientCommandError: TServerCMDSocketCommandErrorEvent read FServerCMDSocketCommandErrorEvent write FServerCMDSocketCommandErrorEvent;
    property OnClientCommandTimeOut: TServerCMDSocketCommandTimeOut read FServerCMDSocketCommandTimeOut write FServerCMDSocketCommandTimeOut;
  end;

procedure Register;

function VarArrayDim(Data: Variant): integer;
function StreamToVariant(Stream: TStream ; Count: int64): Variant;
function ArrayOfByteToVariant(Buf: TArrayOfByte): Variant;
procedure VariantToStream(Data: Variant ; Stream: TStream);
procedure VariantToArrayOfByte(Data: Variant ; var Buf: TArrayOfByte);

implementation

procedure Register;
begin RegisterComponents('TCP/IP', [TClientCMDSocket, TServerCMDSocket]); end;

function VarArrayDim(Data: Variant): integer;
begin result:=VarArrayHighBound(Data, 1)-VarArrayLowBound(Data, 1)+1; end;

function StreamToVariant(Stream: TStream ; Count: int64): Variant;
var MyBuffer: Pointer;
begin
     Count:=Min(Count, Stream.Size-Stream.Position);
     if (Count<=0) or (Count>MAX_ARRAY_SIZE) then VarClear(result)
     else
     begin
          Result:=VarArrayCreate([0, Count-1], VarByte);
          MyBuffer := VarArrayLock(Result);
          Stream.ReadBuffer(MyBuffer^, Count);
          VarArrayUnlock(Result);
     end;
end;

procedure VariantToStream(Data: Variant ; Stream: TStream);
var MyBuffer: Pointer;
begin
     if VarIsArray(Data) and (VarArrayDimCount(Data)=1) then
     begin
          MyBuffer := VarArrayLock(Data);
          Stream.WriteBuffer(MyBuffer^, VarArrayDim(Data));
          VarArrayUnlock(Data);
     end;
end;

function ArrayOfByteToVariant(Buf: TArrayOfByte): Variant;
var MyBuffer: Pointer;
begin
     if (length(Buf)=0) or (length(Buf)>MAX_ARRAY_SIZE) then VarClear(result)
     else
     begin
          Result:=VarArrayCreate([0, length(Buf)-1], VarByte);
          MyBuffer := VarArrayLock(Result);
          CopyMemory(MyBuffer, @Buf[0], length(Buf));
          VarArrayUnlock(Result);
     end;
end;

procedure VariantToArrayOfByte(Data: Variant ; var Buf: TArrayOfByte);
var MyBuffer: Pointer;
begin
     if VarIsArray(Data) and (VarArrayDimCount(Data)=1) then
     begin
          MyBuffer := VarArrayLock(Data);
          Setlength(Buf, VarArrayDim(Data));
          CopyMemory(@Buf[0], MyBuffer, length(Buf));
          VarArrayUnlock(Data);
     end;
end;

function FormatAndSendCmd(Socket: TCustomWinSocket ; Command: AnsiString ; Args: array of Variant ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString;
var i: integer;
    Uid: TGuid;
    Chaine: AnsiString;
    Buf: TArrayOfChar;
    DataInteger: int64;
    DataDouble: double;
    DataBool: bool;
    D: Pointer;
begin
     if Socket.Connected and (Command<>'') then
     try
        if Trim(string(ID))='' then
        begin
             CreateGuid(Uid);
             ID:=AnsiString(GuidToString(Uid));
        end;
        if ID<>'' then
        begin
             Chaine:='';
             for i:=0 to length(Args)-1 do
             begin
                  if VarIsType(Args[i], varUString) then Args[i]:=AnsiString(Args[i]);
                  if VarIsType(Args[i], varString) then Chaine:=Chaine+Arg_Marker[integer(AT_STRING)] else
                  if VarIsType(Args[i], [varByte, varShortInt, varSmallInt, varInteger, varInteger, varLongWord, varInt64]) then Chaine:=Chaine+Arg_Marker[integer(AT_INT64)] else
                  if VarIsType(Args[i], [varSingle, varDouble]) then Chaine:=Chaine+Arg_Marker[integer(AT_DOUBLE)] else
                  if VarIsType(Args[i], varBoolean) then Chaine:=Chaine+Arg_Marker[integer(AT_BOOL)] else
                  if VarIsArray(Args[i]) and (VarArrayDimCount(Args[i])=1) and (SizeOf(AnsiChar)* (VarArrayHighBound( Args[i], 1) - VarArrayLowBound( Args[i], 1) + 1)<=MAX_ARRAY_SIZE) then Chaine:=Chaine+Arg_Marker[integer(AT_ARRAY)]+AnsiString(inttostr(SizeOf(AnsiChar)* (VarArrayHighBound( Args[i], 1) - VarArrayLowBound( Args[i], 1) + 1))) else
                  if VarIsEmpty(Args[i]) then Chaine:=Chaine+Arg_Marker[integer(AT_ARRAY)]+'0';
             end;

             Socket.SendText(Command+EOL+ID+EOL+Reply_ID+EOL+Chaine+EOL);

             for i:=0 to length(Args)-1 do
             begin
                  if VarIsType(Args[i], varString) then
                     Socket.SendText(AnsiString(Args[i])+EOL) else
                  if VarIsType(Args[i], [varByte, varShortInt, varSmallInt, varInteger, varInteger, varLongWord, varInt64]) or VarIsType(Args[i], varInteger)then
                  begin
                       DataInteger:=Args[i];
                       Socket.SendBuf(DataInteger, SizeOf(DataInteger));
                  end
                  else
                  if VarIsType(Args[i], [varSingle, varDouble]) then
                  begin
                       DataDouble:=Args[i];
                       Socket.SendBuf(DataDouble, SizeOf(DataDouble));
                  end
                  else
                  if VarIsType(Args[i], varBoolean) then
                  begin
                       DataBool:=Args[i];
                       Socket.SendBuf(DataBool, SizeOf(DataBool));
                  end
                  else
                  if VarIsArray(Args[i]) and (VarArrayDimCount(Args[i])=1) and (SizeOf(AnsiChar)* (VarArrayHighBound( Args[i], 1) - VarArrayLowBound( Args[i], 1) + 1)<=MAX_ARRAY_SIZE) then
                  try
                     D:=VarArrayLock(Args[i]);
                     try
                        Setlength(Buf, SizeOf(AnsiChar)* (VarArrayHighBound( Args[i], 1) - VarArrayLowBound( Args[i], 1) + 1));
                        CopyMemory(@Buf[0], D, length(Buf));
                     finally
                        VarArrayUnlock(Args[i]);
                     end;
                     Socket.SendBuf(Buf[0], length(Buf));
                  finally
                     Finalize(Buf);
                  end;
             end;
        end;
     except
        ID:='';
     end
     else ID:='';
     for i:=0 to length(Args)-1 do VarClear(Args[i]);
     result:=ID;
end;

/////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  TTimerList //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

constructor TTimerList.Create;
begin
     FWindowHandle := AllocateHWnd(WndProc);
     inherited Create;
end;

destructor TTimerList.Destroy;
begin
     inherited Destroy;
     DeallocateHWnd(FWindowHandle);
end;

procedure TTimerList.WndProc(var Msg: TMessage);
var i: integer;
    Trouve: bool;
begin
     if Msg.Msg = WM_DELETE_TIMER then
     begin
          i:=0; Trouve:=false;
          while (i<Count) and not Trouve do
            if Integer(Items[i])=Integer(Msg.WParam) then Trouve:=true else Inc(i);
          if Trouve then ClearTimers(i);
          Msg.Result := 0;
     end
     else Msg.Result := DefWindowProc(FWindowHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;

procedure TTimerList.ClearTimers(Index: integer=-1);
begin
     if Index=-1 then while Count>0 do ClearTimers(0)
     else
     if (Index>=0) and (Index<Count) then
     begin
          TTimeOutTimer(Items[Index]).Free;
          Delete(Index);
     end;
end;

/////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////  TCommandParser /////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

constructor TCommandParser.Create(ASocket: TCustomWinSocket);
begin
     FWindowHandle := AllocateHWnd(WndProc);
     FSocket:=ASocket;
     Setlength(CommandQueued, 0);
     InitCommand;
     TimerList:=TTimerList.Create;
     inherited Create;
end;

destructor TCommandParser.Destroy;
begin
     inherited Destroy;
     TimerList.ClearTimers;
     TimerList.Free;
     TimerList:=nil;
     InitCommand;
     RemoveCommandQueued;
     DeallocateHWnd(FWindowHandle);
end;

procedure TCommandParser.WndProc(var Msg: TMessage);
begin
     if Msg.Msg = WM_COMMAND then
     begin
          if length(CommandQueued)>0 then
          begin
               OnCommand(Self, Socket, CommandQueued[0]);
               RemoveCommandQueued(0);
          end;
          Msg.Result := 0;
     end
     else Msg.Result := DefWindowProc(FWindowHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;

procedure TCommandParser.RemoveCommandQueued(Index: integer=-1);
var i: integer;
begin
     if Index=-1 then while length(CommandQueued)>0 do RemoveCommandQueued(0)
     else
     if (Index>=0) and (Index<length(CommandQueued)) then
     begin
          CommandQueued[Index].Command:='';
          CommandQueued[Index].ID:='';
          CommandQueued[Index].Reply_ID:='';
          while length(CommandQueued[Index].Argument)>0 do
          begin
               VarClear(CommandQueued[Index].Argument[length(CommandQueued[Index].Argument)-1].Data);
               Setlength(CommandQueued[Index].Argument, length(CommandQueued[Index].Argument)-1);
          end;
          for i:=Index to length(CommandQueued)-2 do CommandQueued[i]:=CommandQueued[i+1];
          Setlength(CommandQueued, length(CommandQueued)-1);
     end;
end;

procedure TCommandParser.InitCommand;
var i: integer;
begin
     FCurrentCommand.Command:='';
     FCurrentCommand.ID:='';
     FCurrentCommand.Reply_ID:='';
     FCurrentCommand.Struct:='';
     for i:=0 to length(FCurrentCommand.Argument)-1 do Finalize(FCurrentCommand.Argument[i].Data);
     Setlength(FCurrentCommand.Argument, 0);
     FCurrentCommand.State:=CS_Command;
     FCurrentCommand.Argument_Index:=0;
end;

// Lit le contenu du Buffer en attente de vidage et analyse son contenu
// pour traiter les commandes compltes sont traites

procedure TCommandParser.Parse(var Buffer: TArrayOfChar);
var i, Position, Depart, DejaLu, ALire: integer;
    LuEOL, TrouveTimer: bool;
    DataInt64: int64;
    DataDouble: double;
    DataBool: bool;
    DataString: Ansistring;
    DataByte: array of byte;
    function VerifierCommande: bool;
    begin
         result:=true;
         OnCommandReceive(Self, Socket, FCurrentCommand, result);
         if not result then InitCommand;
    end;
    function StructureValide: bool;
    var Etat: word;
        j, i: integer;
    begin
         i:=1; result:=true; Etat:=0;
         with FCurrentCommand do
         while result and (i<=length(Struct)) do
         begin
              case Etat of
              0:
              begin
                   if Struct[i]=Arg_Marker[integer(AT_STRING)] then
                   begin
                        Setlength(Argument, length(Argument)+1);
                        Argument[length(Argument)-1].Typ:=AT_STRING;
                        Setlength(Argument[length(Argument)-1].Data, 0);
                        Inc(i);
                   end
                   else
                   if Struct[i]=Arg_Marker[integer(AT_INT64)] then
                   begin
                        Setlength(Argument, length(Argument)+1);
                        Argument[length(Argument)-1].Typ:=AT_INT64;
                        Setlength(Argument[length(Argument)-1].Data, 0);
                        Argument[length(Argument)-1].Size:=SizeOf(Int64);
                        Inc(i);
                   end
                   else
                   if Struct[i]=Arg_Marker[integer(AT_DOUBLE)] then
                   begin
                        Setlength(Argument, length(Argument)+1);
                        Argument[length(Argument)-1].Typ:=AT_DOUBLE;
                        Setlength(Argument[length(Argument)-1].Data, 0);
                        Argument[length(Argument)-1].Size:=SizeOf(Double);
                        Inc(i);
                   end
                   else
                   if Struct[i]=Arg_Marker[integer(AT_BOOL)] then
                   begin
                        Setlength(Argument, length(Argument)+1);
                        Argument[length(Argument)-1].Typ:=AT_BOOL;
                        Setlength(Argument[length(Argument)-1].Data, 0);
                        Argument[length(Argument)-1].Size:=SizeOf(Bool);
                        Inc(i);
                   end
                   else
                   if Struct[i]=Arg_Marker[integer(AT_ARRAY)] then
                   begin
                        Setlength(Argument, length(Argument)+1);
                        Argument[length(Argument)-1].Typ:=AT_ARRAY;
                        Argument[length(Argument)-1].Size:=0;
                        Setlength(Argument[length(Argument)-1].Data, 0);
                        Etat:=1; Inc(i);
                   end
                   else
                   result:=false;
              end;
              1:
              begin
                   if Struct[i] in ['0'..'9'] then
                   with Argument[length(Argument)-1] do
                   begin
                        if Cardinal(Size*10)+StrToInt64(string(Struct[i]))<=MAX_ARRAY_SIZE then
                        begin
                             Size:=Size*10+StrToInt(string(Struct[i]));
                             Inc(i);
                        end
                        else result:=false;
                   end
                   else
                   Etat:=0;
              end;
              end;
         end;
         i:=0;
         with FCurrentCommand do
         while i<length(Argument) do
         begin
              if (Argument[i].Typ=AT_ARRAY) and (Argument[i].Size=0) then
              begin
                   for j:=i to length(Argument)-2 do Argument[j]:=Argument[j+1];
                   Setlength(Argument, length(Argument)-1);
              end else Inc(i);
         end;
         if not result then
         begin
              OnCommandStructError(Self, Socket, FCurrentCommand, result);
              if not result then InitCommand;
         end;
    end;
    procedure Assigner;
    var Chaine: Ansistring;
    begin
         Setlength(Chaine, Position-Depart);
         CopyMemory(@Chaine[1], @Buffer[Depart], Position-Depart);
         with FCurrentCommand do
         case State of
         CS_Command: begin Command:=Command+Chaine; if LuEOL and VerifierCommande then State:=CS_ID; end;
         CS_ID: begin ID:=ID+Chaine; if LuEOL then State:=CS_Reply_ID; end;
         CS_Reply_ID: begin Reply_ID:=Reply_ID+Chaine; if LuEOL then State:=CS_Struct; end;
         CS_Struct: begin Struct:=Struct+Chaine; if LuEOL and StructureValide then begin State:=CS_Argument; Argument_Index:=0; end; end;
         end;
         Finalize(Chaine);
    end;
    procedure AvancerBufferQueue(Datalength: integer);
    var Buf: array of byte;
    begin
         Socket.Lock;
         Setlength(Buf, Datalength);
         recv(Socket.SocketHandle, Buf[0], Datalength, 0);
         Finalize(Buf);
         Socket.UnLock;
    end;
begin
     Depart:=0; Position:=0;
     with FCurrentCommand do
     while Position<length(Buffer) do
     begin
          // Lire jusqu' EOL
          if (State in [CS_Command, CS_ID, CS_Reply_ID, CS_Struct]) or
             ((State=CS_Argument) and (Argument[Argument_Index].Typ=AT_STRING)) then
          begin
               LuEOL:=false;
               while (Position<length(Buffer)) and not LuEOL do
                 if (Position<=length(Buffer)-length(EOL)) and (Buffer[Position]+Buffer[Position+1]=EOL) then LuEOL:=true else Inc(Position);
               if State<>CS_Argument then Assigner else
               begin
                    DejaLu:=length(Argument[Argument_Index].Data);
                    Setlength(Argument[Argument_Index].Data, DejaLu+Position-Depart);
                    CopyMemory(@Argument[Argument_Index].Data[DejaLu], @Buffer[Depart], Position-Depart);
                    Argument[Argument_Index].Size:=length(Argument[Argument_Index].Data);
                    if LuEOL then Inc(Argument_Index);
               end;
               if LuEOL then Inc(Position, length(EOL));
               AvancerBufferQueue(Position-Depart);
               Depart:=Position;
          end
          else
          // Lecture des bits (cas o State=CS_Argument & Typ <> AT_STRING)
          begin
               ALire:=Min(Argument[Argument_Index].Size-length(Argument[Argument_Index].Data), length(Buffer)-Position);
               DejaLu:=length(Argument[Argument_Index].Data);
               Setlength(Argument[Argument_Index].Data, DejaLu+ALire);
               CopyMemory(@Argument[Argument_Index].Data[DejaLu], @Buffer[Depart], ALire);
               AvancerBufferQueue(ALire);
               if length(Argument[Argument_Index].Data)=Argument[Argument_Index].Size then Inc(Argument_Index);
               Position:=Depart+ALire;
               Depart:=Position;
          end;

          if (State=CS_Argument) and (Argument_Index=length(Argument)) then
          begin
               // On stoppe le timer de timeout s'il existait
               i:=0; TrouveTimer:=false; while not TrouveTimer and (i<TimerList.Count) do
               if TTimeOutTimer(TimerList.Items[i]).ID=FCurrentCommand.Reply_ID then TrouveTimer:=true else Inc(i);
               if TrouveTimer then
                  TimerList.ClearTimers(i);

               // Signaler la rception de la commande
               Setlength(CommandQueued, length(CommandQueued)+1);
               CommandQueued[length(CommandQueued)-1].Command:=FCurrentCommand.Command;
               CommandQueued[length(CommandQueued)-1].ID:=FCurrentCommand.ID;
               CommandQueued[length(CommandQueued)-1].Reply_ID:=FCurrentCommand.Reply_ID;
               Setlength(CommandQueued[length(CommandQueued)-1].Argument, length(FCurrentCommand.Argument));
               for i:=0 to length(FCurrentCommand.Argument)-1 do
               begin
                    CommandQueued[length(CommandQueued)-1].Argument[i].Typ:=FCurrentCommand.Argument[i].Typ;
                    case FCurrentCommand.Argument[i].Typ of
                    AT_STRING:
                    begin
                         Setlength(DataString, CurrentCommand.Argument[i].Size);
                         CopyMemory(@DataString[1], @FCurrentCommand.Argument[i].Data[0], FCurrentCommand.Argument[i].Size);
                         CommandQueued[length(CommandQueued)-1].Argument[i].Data:=DataString;
                         Finalize(DataString);
                    end;
                    AT_INT64:
                    begin
                         DataInt64:=0;
                         CopyMemory(@DataInt64, @FCurrentCommand.Argument[i].Data[0], FCurrentCommand.Argument[i].Size);
                         CommandQueued[length(CommandQueued)-1].Argument[i].Data:=DataInt64;
                    end;
                    AT_DOUBLE:
                    begin
                         DataDouble:=0;
                         CopyMemory(@DataDouble, @FCurrentCommand.Argument[i].Data[0], FCurrentCommand.Argument[i].Size);
                         CommandQueued[length(CommandQueued)-1].Argument[i].Data:=DataDouble;
                    end;
                    AT_BOOL:
                    begin
                         DataBool:=false;
                         CopyMemory(@DataBool, @FCurrentCommand.Argument[i].Data[0], FCurrentCommand.Argument[i].Size);
                         CommandQueued[length(CommandQueued)-1].Argument[i].Data:=DataBool;
                    end;
                    AT_ARRAY:
                    begin
                         Setlength(DataByte, FCurrentCommand.Argument[i].Size);
                         CopyMemory(@DataByte[0], @FCurrentCommand.Argument[i].Data[0], FCurrentCommand.Argument[i].Size);
                         CommandQueued[length(CommandQueued)-1].Argument[i].Data:=DataByte;
                         Finalize(DataByte);
                    end;
                    end;
               end;
               PostMessage(FWindowHandle, WM_COMMAND, 0, 0);
               InitCommand;
          end;
     end;
end;

/////////////////////////////////////////////////////////////////////////////////
////////////////////////////////  TClientCmdCollection //////////////////////////
/////////////////////////////////////////////////////////////////////////////////

constructor TClientCMDSocket.Create(AOwner: TComponent);
begin
     FClientSocket := TClientWinSocket.Create(INVALID_SOCKET);
     InitSocket(FClientSocket);
     FCommands:=TStringList.Create;
     TStringList(FCommands).CaseSensitive:=true;
     FClientSocket.ClientType:=ctNonBlocking;
     CommandParser:=TCommandParser.Create(Socket);
     CommandParser.OnCommand:=OnCommandOK;
     CommandParser.OnCommandReceive:=OnCommandReceive;
     CommandParser.OnCommandStructError:=OnCommandStructError;
     FConnectTimer:=TTimer.Create(nil);
     FConnectTimer.Enabled:=false;
     FConnectTimer.OnTimer:=OnTimeOutEvent;
     inherited Create(AOwner);
end;

destructor TClientCMDSocket.Destroy;
begin inherited Destroy; FConnectTimer.Free; FConnectTimer:=nil; FClientSocket.Free; FClientSocket:=nil; FCommands.Free; FCommands:=nil; CommandParser.Free; CommandParser:=nil; end;

procedure TClientCMDSocket.ConnectAndWaitForCommand(TimeOut: cardinal);
begin
     if not Active then
     begin
          Open;
          if TimeOut>0 then
          begin
               FConnectTimer.Enabled:=false;
               FConnectTimer.Interval:=TimeOut;
               FConnectTimer.Enabled:=true;
          end else FConnectTimer.Enabled:=false;
     end;
end;

function TClientCMDSocket.GetCaseSensitive: bool;
begin result:=TStringList(FCommands).CaseSensitive; end;

procedure TClientCMDSocket.SetCaseSensitive(Value: bool);
begin TStringList(FCommands).CaseSensitive:=Value; end;

procedure TClientCMDSocket.SetCommands(Value: TStrings);
begin FCommands.Assign(Value); end;

procedure TClientCMDSocket.OnTimeOutEvent(Sender: TObject);
begin
     if Sender is TTimeOutTimer then
     with TTimeOutTimer(Sender) do
     begin
          Enabled:=false;
          if Assigned(Socket) and Socket.Connected and Assigned(OnCommandTimeOut) then OnCommandTimeOut(self, Command, ID, Interval);
          PostMessage(ParentHandle, WM_DELETE_TIMER, integer(self), 0);
     end
     else
     if Sender=FConnectTimer then
     begin
          FConnectTimer.Enabled:=false;
          if Active and Assigned(FClientCMDSocketConnectTimeOut) then FClientCMDSocketConnectTimeOut(Self, FConnectTimer.Interval);
     end;
end;

procedure TClientCMDSocket.DoActivate(Value: Boolean);
begin
     if (Value <> FClientSocket.Connected) and not (csDesigning in ComponentState) then
     begin
          if FClientSocket.Connected then FClientSocket.Disconnect(FClientSocket.SocketHandle) else
             FClientSocket.Open(Host, Address, Service, Port, ClientType = ctBlocking);
     end;
end;

function TClientCMDSocket.SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString;
var CMDSocketTimer: TTimeOutTimer;
begin
     Command:=AnsiString(Trim(string(Command)));
     if FClientSocket.Connected and (Command<>'') then
     begin
          result:=FormatAndSendCmd(FClientSocket, Command, Args, Reply_ID, ID);
          if (result<>'') and (TimeOut>0) then
          begin
               CMDSocketTimer:=TTimeOutTimer.Create(nil);
               CMDSocketTimer.ID:=result;
               CMDSocketTimer.Command:=Command;
               CMDSocketTimer.Interval:=Timeout;
               CMDSocketTimer.Enabled:=true;
               CMDSocketTimer.ParentHandle:=CommandParser.TimerList.Handle;
               CMDSocketTimer.OnTimer:=OnTimeOutEvent;
               CommandParser.TimerList.Add(CMDSocketTimer);
          end;
     end else result:='';
end;

function TClientCMDSocket.GetClientType: TClientType;
begin Result := FClientSocket.ClientType; end;

procedure TClientCMDSocket.Event(Socket: TCustomWinSocket; SocketEvent: TSocketEvent);
var Buf: TArrayOfChar;
    Datalength: integer;
begin
     case SocketEvent of
     seDisconnect, seConnect: begin if SocketEvent=seDisconnect then FConnectTimer.Enabled:=false; CommandParser.InitCommand; CommandParser.TimerList.ClearTimers; inherited; end;
     seRead:
     begin
          if Socket.Receivelength>0 then
          begin
               Setlength(Buf, Socket.ReceiveLength);
               Datalength:=recv(Socket.SocketHandle, Buf[0], length(Buf), MSG_PEEK);
               if Datalength>0 then
               begin
                    Setlength(Buf, Datalength);
                    CommandParser.Parse(Buf);
               end;
               Finalize(Buf);
          end
          else
          begin Setlength(Buf, 0); Socket.ReceiveBuf(Buf, 0); Finalize(Buf); end;
     end
     else inherited;
     end;
end;

procedure TClientCMDSocket.OnCommandReceive(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool);
var CommandUser: TCommandUser;
begin
     Continue:=FCommands.IndexOf(string(Command.Command))>=0;
     if not Continue {and Socket.Connected} and Assigned(FClientCMDSocketCommandErrorEvent) then
     begin
          CommandUser.Command:=Command.Command;
          CommandUser.ID:='';
          CommandUser.Reply_ID:='';
          Setlength(CommandUser.Argument, 0);
          FClientCMDSocketCommandErrorEvent(Self, CommandUser, CMDSC_Command_Unknown, Continue);
     end;
end;

procedure TClientCMDSocket.OnCommandStructError(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool);
var CommandUser: TCommandUser;
begin
     if not Continue {and Socket.Connected} and Assigned(FClientCMDSocketCommandErrorEvent) then
     begin
          CommandUser.Command:=Command.Command;
          CommandUser.ID:=Command.ID;
          CommandUser.Reply_ID:=Command.Reply_ID;
          Setlength(CommandUser.Argument, 0);
          FClientCMDSocketCommandErrorEvent(Self, CommandUser, CMDSC_Illegal_Parameter, Continue);
     end;
end;

procedure TClientCMDSocket.OnCommandOK(Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommandUser);
begin
     FConnectTimer.Enabled:=false;
     if {Socket.Connected and} Assigned(FClientCMDSocketCommandEvent) then FClientCMDSocketCommandEvent(Self, CommandEx);
end;

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  TServerWinCMDSocket //////////////////////////
/////////////////////////////////////////////////////////////////////////////////

function TServerWinCMDSocket.GetConnections(Index: Integer): TServerClientCMDSocket;
begin Result := TServerClientCMDSocket(inherited Connections[Index]); end;

/////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////  TServerCMDSocket ////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

constructor TServerCMDSocket.Create(AOwner: TComponent);
begin
     FServerSocket := TServerWinCMDSocket.Create(INVALID_SOCKET);
     InitSocket(FServerSocket);
     FServerSocket.ThreadCacheSize := 10;
     FServerSocket.ServerType:=stNonBlocking;
     OnGetSocket:=GetSocket;
     FCommands:=TStringList.Create;
     TStringList(FCommands).CaseSensitive:=true;
     inherited Create(AOwner);
end;

destructor TServerCMDSocket.Destroy;
begin
     inherited Destroy;
     FCommands.Free;
     FCommands:=nil;
end;

function TServerCMDSocket.GetCaseSensitive: bool;
begin result:=TStringList(FCommands).CaseSensitive; end;

procedure TServerCMDSocket.SetCaseSensitive(Value: bool);
begin TStringList(FCommands).CaseSensitive:=Value; end;

procedure TServerCMDSocket.SetCommands(Value: TStrings);
begin FCommands.Assign(Value); end;

function TServerCMDSocket.GetServerType: TServerType;
begin Result := FServerSocket.ServerType; end;

procedure TServerCMDSocket.GetSocket(Sender: TObject; ASocket: TSocket; var ClientSocket: TServerClientWinSocket);
begin ClientSocket:=TServerClientCMDSocket.Create(self, ASocket, FServerSocket); end;

function TServerCMDSocket.GetServerSocket: TServerWinCMDSocket;
begin result:=TServerWinCMDSocket(FServerSocket); end;

/////////////////////////////////////////////////////////////////////////////////
///////////////////////////////  TServerClientCMDSocket /////////////////////////
/////////////////////////////////////////////////////////////////////////////////

constructor TServerClientCMDSocket.Create(AOwner: TServerCMDSocket; Socket: TSocket; ServerWinSocket: TServerWinSocket);
begin
     FOwner:=AOwner;
     CommandParser:=TCommandParser.Create(self);
     CommandParser.OnCommand:=OnCommandOK;
     CommandParser.OnCommandReceive:=OnCommandReceive;
     CommandParser.OnCommandStructError:=OnCommandStructError;
     inherited Create(Socket, ServerWinSocket);
end;

destructor TServerClientCMDSocket.Destroy;
begin
     inherited Destroy;
     CommandParser.Free;
     CommandParser:=nil;
end;

procedure TServerClientCMDSocket.OnCommandReceive(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool);
var CommandUser: TCommandUser;
begin
     Continue:=FOwner.FCommands.IndexOf(string(Command.Command))>=0;
     if not Continue and Assigned(FOwner.OnClientCommandError) then
     begin
          CommandUser.Command:=Command.Command;
          CommandUser.ID:='';
          CommandUser.Reply_ID:='';
          Setlength(CommandUser.Argument, 0);
          FOwner.OnClientCommandError(FOwner, self, CommandUser, CMDSC_Command_Unknown, Continue);
     end;
end;

procedure TServerClientCMDSocket.OnCommandStructError(Sender: TObject ; Socket: TCustomWinSocket; Command: TCommand ; var Continue: bool);
var CommandUser: TCommandUser;
begin
     if not Continue and Assigned(FOwner.OnClientCommandError) then
     begin
          CommandUser.Command:=Command.Command;
          CommandUser.ID:=Command.ID;
          CommandUser.Reply_ID:=Command.Reply_ID;
          Setlength(CommandUser.Argument, 0);
          FOwner.OnClientCommandError(FOwner, self, CommandUser, CMDSC_Illegal_Parameter, Continue);
     end;
end;

procedure TServerClientCMDSocket.OnCommandOK(Sender: TObject ; Socket: TCustomWinSocket; CommandEx: TCommandUser);
begin
     if Assigned(FOwner.OnClientCommand) then FOwner.OnClientCommand(FOwner, self, CommandEx);
end;

procedure TServerClientCMDSocket.Event(Socket: TCustomWinSocket; SocketEvent: TSocketEvent);
var Buf: TArrayOfChar;
    Datalength: integer;
begin
     case SocketEvent of
     seDisconnect, seConnect: begin CommandParser.InitCommand; CommandParser.TimerList.ClearTimers; inherited; end;
     seRead:
     begin
          if Socket.Receivelength>0 then
          begin
               Setlength(Buf, Socket.ReceiveLength);
               Datalength:=recv(Socket.SocketHandle, Buf[0], length(Buf), MSG_PEEK);
               if Datalength>0 then
               begin
                    Setlength(Buf, Datalength);
                    CommandParser.Parse(Buf);
               end;
               Finalize(Buf);
          end
          else
          begin Setlength(Buf, 0); Socket.ReceiveBuf(Buf, 0); Finalize(Buf); end;
     end
     else inherited;
     end;
end;

function TServerClientCMDSocket.SendCommand(Command: AnsiString; Args: array of Variant ; TimeOut: cardinal=0 ; Reply_ID: AnsiString='' ; ID: AnsiString=''): AnsiString;
var CMDSocketTimer: TTimeOutTimer;
begin
     Command:=AnsiString(Trim(string(Command)));
     if Command<>'' then
     begin
          result:=FormatAndSendCmd(self, Command, Args, Reply_ID, ID);
          if (result<>'') and (TimeOut>0) then
          begin
               CMDSocketTimer:=TTimeOutTimer.Create(nil);
               CMDSocketTimer.ID:=result;
               CMDSocketTimer.Command:=Command;
               CMDSocketTimer.Interval:=Timeout;
               CMDSocketTimer.Enabled:=true;
               CMDSocketTimer.ParentHandle:=CommandParser.TimerList.Handle;
               CMDSocketTimer.OnTimer:=OnTimeOutEvent;
               CommandParser.TimerList.Add(CMDSocketTimer);
          end;
     end else result:='';
end;

procedure TServerClientCMDSocket.OnTimeOutEvent(Sender: TObject);
begin
     if Sender is TTimeOutTimer then
     with TTimeOutTimer(Sender) do
     begin
          Enabled:=false;
          if (Self.Connected) and Assigned(FOwner.OnClientCommandTimeOut) then FOwner.OnClientCommandTimeOut(FOwner, self, Command, ID, Interval);
          PostMessage(ParentHandle, WM_DELETE_TIMER, integer(self), 0);
     end;
end;

end.
